import { spawn } from 'node:child_process';
import { loadEnv } from 'vite';
import makeDebugger from 'debug';
import c from 'picocolors';
import { toArray } from '@antfu/utils';

const PLUGIN_NAME = "vite:plugin:run";
const debug = makeDebugger(PLUGIN_NAME);
function warn(prefix, message) {
  process.stdout.write(c.bold(`${c.yellow(`(!) ${prefix}`)} ${message}
`));
}
function run(options = []) {
  const silent = "silent" in options ? options.silent : void 0;
  const skipDts = "skipDts" in options ? options.skipDts : void 0;
  const resolvedOptions = {
    env: {},
    silent: silent ?? true,
    skipDts: skipDts ?? true,
    input: toArray("input" in options ? options.input : options)
  };
  return {
    name: PLUGIN_NAME,
    configResolved(config) {
      resolvedOptions.env = loadEnv(config.mode ?? process.env.NODE_ENV ?? "development", process.cwd(), "");
      debug("Given options:", options);
      debug("Resolved options:", resolvedOptions);
      resolvedOptions.input.forEach((runner) => {
        if (runner.startup !== false || runner.condition === void 0) {
          handleRunnerCommand(resolvedOptions, runner);
        }
      });
    },
    configureServer(server) {
      server.watcher.on("add", (path) => handleReload(resolvedOptions, { file: path, server, type: "add" })).on("change", (path) => handleReload(resolvedOptions, { file: path, server, type: "change" })).on("unlink", (path) => handleReload(resolvedOptions, { file: path, server, type: "unlink" }));
    },
    handleHotUpdate({ file }) {
      if (resolvedOptions.skipDts && file.endsWith(".d.ts")) {
        return [];
      }
    }
  };
}
function handleReload(options, parameters) {
  const file = parameters.file.replaceAll("\\", "/");
  options.input.forEach((runner) => {
    if (!runner.condition?.(file)) {
      return;
    }
    debug(`${c.gray(file)} changed, applying its handler...`);
    handleRunner(runner, options, parameters);
  });
}
function handleRunner(runner, options, parameters) {
  try {
    if (typeof runner.onFileChanged === "function") {
      runner.onFileChanged?.(parameters);
    }
    if (Array.isArray(runner.run)) {
      handleRunnerCommand(options, runner);
    }
  } catch (error) {
    warn(PLUGIN_NAME, `Handler failed for ${parameters.file}: ${error.message}`);
    debug("Full error:", error);
  }
}
function handleRunnerCommand(options, runner) {
  if (!runner.run) {
    return;
  }
  const name = getRunnerName(runner);
  debug(`Running ${name}...`);
  setTimeout(() => {
    const child = spawn(getExecutable(options, getRunnerCommand(runner)), getRunnerArguments(runner), { shell: true });
    if (!options.silent) {
      child.stdout.on("data", (data) => {
        process.stdout.write(data.toString());
      });
      child.stderr.on("data", (data) => {
        process.stdout.write(data.toString());
      });
    }
    child.on("close", (code) => {
      const result = code === 0 ? "ran successfully" : `failed with code ${code}`;
      debug(`${name} ${result}`);
    });
  }, runner.delay ?? 50);
}
function getRunnerName(runner) {
  return c.bold(`[${`${runner.name}` ?? getRunnerCommand(runner) ?? "<runner>"}]`);
}
function getRunnerArguments(runner) {
  const args = typeof runner.run === "function" ? runner.run() : runner.run ?? [];
  return args.slice(1) ?? [];
}
function getRunnerCommand(runner) {
  const args = typeof runner.run === "function" ? runner.run() : runner.run ?? [];
  return args?.[0];
}
function getExecutable(options, name) {
  if (!name) {
    throw new Error("No executable given.");
  }
  return process.env[`${name}_PATH`] || options.env[`${name}_PATH`] || name;
}

export { run as default, run };
